#SHADER_VERSION

in vec4 PARTICLE_DATA;
in vec2 PARTICLE_SETTINGS;


out vec4 OUTPUT;
out vec2 OUTPUT_SETTINGS;



uniform sampler2D dirs;
uniform vec3 dt_RndSeed_SpeedFactor;

uniform vec4 visibleArea;
uniform vec2 offset;


#my_include "ventusky/ventusky_layer_reprojection.ps.glsl"

//http://aras-p.info/blog/2009/07/30/encoding-floats-to-rgba-the-final/
//http://www.ozone3d.net/blogs/lab/20080604/glsl-float-to-rgba8-encoder/


float rand(vec2 co)
{
	float a = 12.9898;
	float b = 78.233;
	float c = 43758.5453;
	float dt = dot(co.xy, vec2(a, b));
	float sn = mod(dt, 3.14);
	return fract(sin(sn) * c);
}


/// <summary>
/// Lieanr mapping of single float from one interval to another
/// </summary>
float MapRange(float fromMin, float fromMax, float toMin, float toMax, float s)
{
	return toMin + (s - fromMin) * (toMax - toMin) / (fromMax - fromMin);
}

/// <summary>
/// Lieanr mapping of vec2 from one interval to another
/// </summary>
vec2 MapRangeV2(vec2 fromMin, vec2 fromMax, vec2 toMin, vec2 toMax, vec2 s)
{
	return toMin + (s - fromMin) * (toMax - toMin) / (fromMax - fromMin);
}

/// <summary>
/// Reinit particle after its death or after it leaves area
/// </summary>
vec4 RebornParticle(vec4 old)
{
	
	vec4 p = vec4(0.0);
	p.x = rand(old.xy * dt_RndSeed_SpeedFactor.y);
	p.y = rand(old.yx * dt_RndSeed_SpeedFactor.y);
	
	p.z = 0.0;
	p.w = old.w;
	
	
	return p;
}

/// <summary>
/// Update single particle
/// </summary>
vec4 UpdateParticle(vec4 p, float lifeTime)
{
	
	vec2 dirTexCoord = MapRangeV2(vec2(0.0), vec2(1.0), visibleArea.xy, visibleArea.zw, p.xy);
	//dirTexCoord.y = 1.0 - dirTexCoord.y;

	dirTexCoord.x = mod(dirTexCoord.x, 1.0);
	
	vec2 reprojTx = FromEquirectangularToMercator(dirTexCoord);
	        
	dirTexCoord = reprojTx;

	vec2 d = texture(dirs, dirTexCoord).rg;


	//DEBUG
	//d.x = 0.5;
	//d.x = d.y;

	//direction must be moved to "-1, 1" (or here (-0.5, 0.5))
	d -= vec2(0.5, 0.5);


	//move particle and update time	
	p.xy += d * dt_RndSeed_SpeedFactor.z * dt_RndSeed_SpeedFactor.x;

	//apply movement offset to prevent "shower door" 
	//this is applied only during frame movement, 
	//otherwise vec2(0,0) is used
	p.xy += offset;



	//update timings
	p.z += dt_RndSeed_SpeedFactor.x;
	p.w = lifeTime;

	return p;
}

/// <summary>
/// main method
/// </summary>
void main()
{
	vec4 p = vec4(0);

	p.xy = PARTICLE_DATA.xy;
	p.zw = PARTICLE_SETTINGS.xy;
	vec2 lastP = PARTICLE_DATA.xy + offset;

	if (p.z > p.w)
	{
		//timeout or not inited - reinit particle

		p = RebornParticle(p);
		lastP = p.xy;
		
	}
	else 
	{
		if ((p.x > 1.0) || (p.x < 0.0) || 
			(p.y > 1.0) || (p.y < 0.0))
		{
			p = RebornParticle(p);
			lastP = p.xy;
		}
		else
		{			
			p = UpdateParticle(p, p.w);			
		}

			
	}
	//write output
	
	OUTPUT.xy = p.xy;
	OUTPUT.zw = lastP;
	OUTPUT_SETTINGS = p.zw;

}

